/**
 * Ce sketch Arduino est destin  tre utilis avec une tlcommande sur liaison srie
 * qui envoit sur la liaison srie 3 octets : 
 * - 0xFF
 * - vitesse gauche (entre -100 et +100)
 * - vitesse droite (entre -100 et +100)
 * 
 * En retour, l'Arduino envoie sur la liaison srie les valeurs d'odomtrie
 */

#include <Servo.h>
#include <MsTimer2.h>

// constantes indiquant la position (en degrs) neutre (roue immobile)
//  ajuster pour chacune des roues et chacun des robots
#define NEUTRE_G  76
#define NEUTRE_D  77

// nombre de vitesses autour du neutre
#define VAR  5

Servo servG;
Servo servD;

// vitesses (entre -100 et 100)
int vg;
int vd;

// encodeurs
long countG;
long countD;

void setup()
{
  // communication srie
  Serial.begin(115200);
  Serial.println("> Connection to Pobot Easy granted.");
  // connecter les servomoteurs
  servG.attach(10);
  servD.attach(9);
  // arrter le robot
  servG.write(NEUTRE_G);
  servD.write(NEUTRE_D);
  // initialiser les vitesses
  vg = 0;
  vd = 0;
  // dclarer les interruptions externes
  attachInterrupt(0,gauche,CHANGE);
  attachInterrupt(1,droite,CHANGE);  
  // grer une interruption rgulire
   MsTimer2::set(250, feedback); // 500ms period
   MsTimer2::start();
}

void loop()
{
  // mise  jour des vitesses
  if (Serial.available() >= 3)
  {    
    if (Serial.read() != 0xFF) 
    {
      Serial.println("Erreur de rception !!");
      Serial.flush();
      return;      
    }
    vg = Serial.read();
    vd = Serial.read();
    // rtablir les valeurs signes
    if (vg >= 128) {
      vg = vg-255;      
    }   
    if (vd >= 128) {
      vd = vd-255;  
    }
    // commander les servomoteurs  
    servG.write(NEUTRE_G+vg*VAR/100.0);
    servD.write(NEUTRE_D-vd*VAR/100.0);
    Serial.flush();
  }
  delay(20);
}

/** Gestion de la communication en retour  **/

// Envoyer une trame d'informations sur la liaison srie
void feedback()
{
  Serial.print(vg,DEC);
  Serial.print(" ");
  Serial.print(vd,DEC);
  Serial.print(" ");
  Serial.print(countG,DEC);
  Serial.print(" ");
  Serial.print(countD,DEC);
  // TODO: les capteurs par exemple
  Serial.println();
}

/** Gestion de l'odomtrie **/

// Interruption de l'odomtre gauche
void gauche()
{
  // Gestion du sens de rotation
  if (vg <0) {
    countG--;
  } 
  else {
    countG++; 
  } 
}

// Interruption de l'odomtre droit
void droite()
{
  // Gestion du sens de rotation
  if (vd <0) {
    countD--;   
  } 
  else {
    countD++;
  }
}

